# -*- coding: utf-8 -*-

__all__ = [
    'HttpException',
    'EmptyResponseWarning',
    'RedirectionException',
    'BadRequestException',
    'UnauthorizedException',
    'ResourceNotFoundException',
    'ResourceConflictException',
    'RequestEntityTooLargeException',
    'ClientRequestException',
    'ServerException',
    'AuthorizeErrorException'
]


class AuthorizeErrorException(Exception):
    def __init__(self, msg, error, error_description):
        super(AuthorizeErrorException, self).__init__(msg)
        self.error = error
        self.error_description = error_description


class HttpException(Exception):
    """
    Class for representing http errors. Contains the response.
    """
    def __init__(self, msg, res):
        super(Exception, self).__init__(msg)
        self.response = res

    @property
    def status_code(self):
        return self.response.status_code

    @property
    def headers(self):
        return self.response.headers

    @property
    def content(self):
        return self.response.content


# 204
class EmptyResponseWarning(HttpException):
    pass


# 3xx codes
class RedirectionException(HttpException):
    pass


# 400
class BadRequestException(HttpException):
    pass


# 401
class UnauthorizedException(HttpException):
    pass


# 404
class ResourceNotFoundException(HttpException):
    pass


# 409
class ResourceConflictException(HttpException):
    pass


# 413
class RequestEntityTooLargeException(HttpException):
    pass


# 4xx codes
class ClientRequestException(HttpException):
    pass


# 5xx codes
class ServerException(HttpException):
    pass
